using System;
using Server.Items;

namespace Server.Items
{
	public abstract class BasePowder : Item, ICommodity
	{
        private CraftResource m_Resource;

        [CommandProperty( AccessLevel.GameMaster )]
		public CraftResource Resource
		{
			get{ return m_Resource; }
			set{ m_Resource = value; InvalidateProperties(); }
		}

        string ICommodity.Description
        {
            get
            {
                return String.Format(Amount == 1 ? "{0} {1} Powder" : "{0} {1} Powders", Amount, CraftResources.GetName(m_Resource).ToLower());
            }
        }

        public abstract BasePowder GetPowders();

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 1 ); // version THIS WAS ONE TO START WITH
			writer.Write( (int) m_Resource );
		}

        public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_Resource = (CraftResource)reader.ReadInt();
					break;
				}	
                case 0:
				{
					OreInfo info;

					switch ( reader.ReadInt() )
					{
						case 0: info = OreInfo.Iron; break;
						
						default: info = null; break;
					}

					m_Resource = CraftResources.GetFromOreInfo( info );
					break;
				}
			}
		}

        public BasePowder ( CraftResource resource ) : this( resource, 1 )
		{
		}

		public BasePowder ( CraftResource resource, int amount ) : base( 0x26B8 ) // 0x3194
		{
			Stackable = true;
			Weight = 0.2;
			Amount = amount;
			Hue = CraftResources.GetHue( resource );

			m_Resource = resource;
		}

        public BasePowder(Serial serial) : base(serial)
		{
		}

        public override void AddNameProperty( ObjectPropertyList list )
		{   
            if (Amount > 1)
                list.Add(1050039, "{0}\t#{1}", Amount, 1075802); // ~1_NUMBER~ ~2_ITEMNAME~
            else
                list.Add(1075802);
		}

        public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );

			if ( !CraftResources.IsStandard( m_Resource ) )
			{
				int num = CraftResources.GetLocalizationNumber( m_Resource );

				if ( num > 0 )
					list.Add( num );
				else
					list.Add( CraftResources.GetName( m_Resource ) );
			}
		}

        public override int LabelNumber
		{
			get
			{
                if (m_Resource >= CraftResource.IronPowder && m_Resource <= CraftResource.EtheralPowder)
                    return 1042845 + (int)(m_Resource - CraftResource.IronPowder);

                return 1044495; // iron ore;
			}
		}
    }

    // POWDER TYPES START HERE

    public class Powder : BasePowder
	{
		[Constructable]
		public Powder() : this( 1 )
		{
		}

		[Constructable]
        public Powder(int amount) : base(CraftResource.RegularPowder, amount)
		{
		}

        public Powder(Serial serial) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}

        public override BasePowder GetPowders()
        {
            return new Powder ();
        }
	}

    public class IronPowder : BasePowder
    {
        [Constructable]
        public IronPowder()
            : this(1)
        {
        }

        [Constructable]
        public IronPowder(int amount) : base(CraftResource.IronPowder, amount)
        {
        }

        public IronPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new IronPowder();
        }
    }

    public class DullCopperPowder : BasePowder
    {
        [Constructable]
        public DullCopperPowder() : this(1)
        {
        }

        [Constructable]
        public DullCopperPowder(int amount) : base(CraftResource.DullCopperPowder, amount)
        {
        }

        public DullCopperPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new DullCopperPowder();
        }
    }

    public class ShadowIronPowder : BasePowder
    {
        [Constructable]
        public ShadowIronPowder() : this(1)
        {
        }

        [Constructable]
        public ShadowIronPowder(int amount) : base(CraftResource.ShadowIronPowder, amount)
        {
        }

        public ShadowIronPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new ShadowIronPowder();
        }
    }

    public class CopperPowder : BasePowder
    {
        [Constructable]
        public CopperPowder() : this(1)
        {
        }

        [Constructable]
        public CopperPowder(int amount) : base(CraftResource.CopperPowder, amount)
        {
        }

        public CopperPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new CopperPowder();
        }
    }

    public class BronzePowder : BasePowder
    {
        [Constructable]
        public BronzePowder() : this(1)
        {
        }

        [Constructable]
        public BronzePowder(int amount) : base(CraftResource.BronzePowder, amount)
        {
        }

        public BronzePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new BronzePowder();
        }
    }

    public class GoldPowder : BasePowder
    {
        [Constructable]
        public GoldPowder() : this(1)
        {
        }

        [Constructable]
        public GoldPowder(int amount) : base(CraftResource.GoldPowder, amount)
        {
        }

        public GoldPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new GoldPowder();
        }
    }

    public class SilverPowder : BasePowder
    {
        [Constructable]
        public SilverPowder() : this(1)
        {
        }

        [Constructable]
        public SilverPowder(int amount) : base(CraftResource.SilverPowder, amount)
        {
        }

        public SilverPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new SilverPowder();
        }
    }

    public class AgapitePowder : BasePowder
    {
        [Constructable]
        public AgapitePowder() : this(1)
        {
        }

        [Constructable]
        public AgapitePowder(int amount) : base(CraftResource.AgapitePowder, amount)
        {
        }

        public AgapitePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new AgapitePowder();
        }
    }

    public class VeritePowder : BasePowder
    {
        [Constructable]
        public VeritePowder() : this(1)
        {
        }

        [Constructable]
        public VeritePowder(int amount) : base(CraftResource.VeritePowder, amount)
        {
        }

        public VeritePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new VeritePowder();
        }
    }

    public class ValoritePowder : BasePowder
    {
        [Constructable]
        public ValoritePowder() : this(1)
        {
        }

        [Constructable]
        public ValoritePowder(int amount) : base(CraftResource.ValoritePowder, amount)
        {
        }

        public ValoritePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new ValoritePowder();
        }
    }

    public class UridiumPowder : BasePowder
    {
        [Constructable]
        public UridiumPowder() : this(1)
        {
        }

        [Constructable]
        public UridiumPowder(int amount) : base(CraftResource.UridiumPowder, amount)
        {
        }

        public UridiumPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new UridiumPowder();
        }
    }

    public class TrilliumPowder : BasePowder
    {
        [Constructable]
        public TrilliumPowder() : this(1)
        {
        }

        [Constructable]
        public TrilliumPowder(int amount) : base(CraftResource.TrilliumPowder, amount)
        {
        }

        public TrilliumPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new TrilliumPowder();
        }
    }

    public class TitaniumPowder : BasePowder
    {
        [Constructable]
        public TitaniumPowder() : this(1)
        {
        }

        [Constructable]
        public TitaniumPowder(int amount) : base(CraftResource.TitaniumPowder, amount)
        {
        }

        public TitaniumPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new TitaniumPowder();
        }
    }

    public class PlatinumPowder : BasePowder
    {
        [Constructable]
        public PlatinumPowder() : this(1)
        {
        }

        [Constructable]
        public PlatinumPowder(int amount) : base(CraftResource.PlatinumPowder, amount)
        {
        }

        public PlatinumPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new PlatinumPowder();
        }
    }

    public class ZenitePowder : BasePowder
    {
        [Constructable]
        public ZenitePowder() : this(1)
        {
        }

        [Constructable]
        public ZenitePowder(int amount) : base(CraftResource.ZenitePowder, amount)
        {
        }

        public ZenitePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new ZenitePowder();
        }
    }

    public class NaquinitePowder : BasePowder
    {
        [Constructable]
        public NaquinitePowder() : this(1)
        {
        }

        [Constructable]
        public NaquinitePowder(int amount) : base(CraftResource.NaquinitePowder, amount)
        {
        }

        public NaquinitePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new NaquinitePowder();
        }
    }

    public class GalvinitePowder : BasePowder
    {
        [Constructable]
        public GalvinitePowder() : this(1)
        {
        }

        [Constructable]
        public GalvinitePowder(int amount) : base(CraftResource.GalvinitePowder, amount)
        {
        }

        public GalvinitePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new GalvinitePowder();
        }
    }

    public class TrilamidePowder : BasePowder
    {
        [Constructable]
        public TrilamidePowder() : this(1)
        {
        }

        [Constructable]
        public TrilamidePowder(int amount) : base(CraftResource.TrilamidePowder, amount)
        {
        }

        public TrilamidePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new TrilamidePowder();
        }
    }

    public class VeramidePowder : BasePowder
    {
        [Constructable]
        public VeramidePowder() : this(1)
        {
        }

        [Constructable]
        public VeramidePowder(int amount) : base(CraftResource.VeramidePowder, amount)
        {
        }

        public VeramidePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new VeramidePowder();
        }
    }

    public class ZenlamidePowder : BasePowder
    {
        [Constructable]
        public ZenlamidePowder() : this(1)
        {
        }

        [Constructable]
        public ZenlamidePowder(int amount) : base(CraftResource.ZenlamidePowder, amount)
        {
        }

        public ZenlamidePowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new ZenlamidePowder();
        }
    }

    public class EtheralPowder : BasePowder
    {
        [Constructable]
        public EtheralPowder() : this(1)
        {
        }

        [Constructable]
        public EtheralPowder(int amount) : base(CraftResource.EtheralPowder, amount)
        {
        }

        public EtheralPowder(Serial serial) : base(serial)
        {
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }

        public override BasePowder GetPowders()
        {
            return new EtheralPowder();
        }
    }
}